package com.family168.jbpm4book;

import junit.framework.*;
import org.jbpm.api.*;
import org.jbpm.api.cmd.*;
import org.jbpm.pvm.internal.migration.*;
import org.jbpm.test.*;
import org.hibernate.*;


public class MigrateTest extends JbpmTestCase {
    public String deployJpdlClasspath(String path) {
        String deploymentDbid = repositoryService.createDeployment()
                                                 .addResourceFromClasspath(path)
                                                 .deploy();

        this.registerDeployment(deploymentDbid);

        return deploymentDbid;
    }

    public void testMigrateRunning() {
        deployJpdlClasspath("jpdl/a.jpdl.xml");

        ProcessInstance processInstance = executionService
            .startProcessInstanceById("a-1");
        assertTrue(processInstance.isActive("a"));

        String id = processInstance.getId();
        processInstance = executionService.signalExecutionById(id);
        assertTrue(processInstance.isActive("x"));

        deployJpdlClasspath("jpdl/b.jpdl.xml");

        processInstance = executionService.signalExecutionById(id);
        assertTrue(processInstance.isActive("b"));
        processInstance = executionService.signalExecutionById(id);
        assertTrue(processInstance.isEnded());
    }

    public void testMigrateRunningIntern() {
        deployJpdlClasspath("jpdl/a.jpdl.xml");

        ProcessInstance processInstance = executionService
            .startProcessInstanceById("a-1");
        assertTrue(processInstance.isActive("a"));

        String id = processInstance.getId();
        processInstance = executionService.signalExecutionById(id);
        assertTrue(processInstance.isActive("x"));

        String deploymentId = deployJpdlClasspath("jpdl/c.jpdl.xml");

        // Get new process definition
        ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery()
                                                               .deploymentId(deploymentId)
                                                               .uniqueResult();
        assertTrue(processDefinition.getName().equalsIgnoreCase("c"));

        final ProcessDefinition pd = processDefinition;
        final ProcessInstance pi = processInstance;
        // Migrate instance here
        processEngine.execute(new Command() {
            public Object execute(Environment env) {
                new DefaultMigrationHandler().migrateInstance(pd,
                    pi, new MigrationDescriptor());
                env.get(Session.class).update(pi);
                return null;
            }
        });

        processInstance = executionService.signalExecutionById(id);
        assertTrue(processInstance.isActive("b"));
        processInstance = executionService.signalExecutionById(id);
        assertTrue(processInstance.isEnded());
    }
}
